from flask import Flask,render_template,request
from flask_session import Session
import os,time
import datetime
from datetime import timedelta,date,datetime
import traceback
import logging

app = Flask(__name__)


app.config.from_object('core.config.SECRET_KEY')
app.config.from_object('core.config.ProductionConfig')

Session(app)

os.environ['TZ'] ='Asia/Calcutta'
time.tzset()

config = app.config

# @app.before_request 
# def before_request_callback(): 
# 	# 	return "before call"
# 	x = request.host.split(".")
# 	if len(x) > 1:
# 		config["SESSION_COOKIE_DOMAIN"] = str("."+x[1]+"."+x[2])


from core.library.cryptography import Cryptography
from core.library.auth import Auth
from core.library.route_group import RouteGroup
from core.library.helper import Helper
from core.model.Log import Log
from core.library.csvexport import CSVEXPORT

current_app = app

#from core import routes
from core.controller.UserController import app as user
app.register_blueprint(user, url_prefix='')

from core.controller.BOController import app as bo
app.register_blueprint(bo, url_prefix='')

class SQLAlchemyHandler(logging.Handler):

    def emit(self, record):
        trace = None
        exc = record.__dict__['exc_info']
        if exc:
            trace = traceback.format_exc()

        path = request.path
        method = request.method
        ip = request.remote_addr    

        data = {
                    'url'     :path,
                'logger_name' :record.__dict__['name'],
                'level'       :record.__dict__['levelname'],
                'context'     :trace,
                'message'     :record.__dict__['msg'],
                'created_at'  :datetime.now(),
                'ip_address'  :ip,
                'app_type'    : Helper.app_type,
                'host_url'    : request.host
            }    
        Log().insert(data)    
        

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

ch = SQLAlchemyHandler()
ch.setLevel(logging.INFO)

formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
ch.setFormatter(formatter)


loggers = [logger, logging.getLogger('sqlalchemy'), logging.getLogger('flask.app')]

for l in loggers:
    l.addHandler(ch)