from flask import Flask, request, Blueprint, jsonify, json,redirect, url_for,flash, render_template,session, app
# import collections
import boto3, botocore
import time,random
import ast
import os
import base64
import io
import logging,requests

from datetime import timedelta,date,datetime
from werkzeug.utils import secure_filename
from os.path import join, dirname, realpath
from botocore.exceptions import ClientError
from urllib.parse import unquote
from PIL import Image

# Numerotec libraries
from .. import Cryptography,Auth,RouteGroup,Log,Helper
from core.library.email import EMAIL
# Models
from core.model.UserModel import UserModel

app = Blueprint('user', __name__)

@app.route('/',methods = ["GET", "POST"])
def Home_index():
	host      = request.host
	host_data = UserModel().get_society(None,None,host)
	if host_data:
		return redirect(url_for("user.Home",society_id=host_data["society_id"],society_key=host_data["society_key"]))
	else:
		return "Application not found"

@app.route('/<int:society_id>/<society_key>', methods = ["GET", "POST"])
@RouteGroup.society_required
@RouteGroup.app_lock_requried
def Home(society_id,society_key):
	if Auth.check(society_id):
		# encrypt_user_id = Cryptography.encrypt(int(Auth.user(society_id).get('user_id')))
		return redirect(url_for('user.getProfile',society_id=society_id,society_key=society_key))
	else:
		return redirect(url_for('user.getLogin',society_id=society_id,society_key=society_key))

@app.route('/<int:society_id>/<society_key>/app_lock', methods = ["GET", "POST"])
@RouteGroup.society_required
def AppLock(society_id,society_key):
	societyData     = Auth.getSociety(society_id)
	return render_template('users/app_lock_message.html',society_id=society_id,society_key=society_key,societyData=societyData)
	
""" Login and logout functions """
@app.route('/<int:society_id>/<society_key>/login')
@RouteGroup.society_required
@RouteGroup.app_lock_requried
def getLogin(society_id,society_key):
	if Auth.check(society_id):
		encrypt_user_id = Cryptography.encrypt(int(Auth.user(society_id).get('user_id')))
		return redirect(url_for('user.getProfile',society_id=society_id,society_key=society_key))
	else:
		societyData     = Auth.getSociety(society_id)
		contents     =  UserModel().get_society_contents(society_id) #get fresh data from database 
		# return societyData
		datas ={'html_content': contents['index_content'],'societyData':societyData}
		return render_template('users/login.html',society_id=society_id,society_key=society_key,Data=societyData,datas=datas)


@app.route('/<int:society_id>/<society_key>/login',methods = ["POST"])
@RouteGroup.society_required
@RouteGroup.app_lock_requried
def postLogin(society_id,society_key):
	if request.method == "POST":
		email               = request.form['email']
		societyData         = Auth.getSociety(society_id)
		profile_member_type = societyData['profile_member_type']
		if  not email :
			flash('Please enter your registered email ','errorMsg')
			return redirect (url_for('user.getLogin',society_id=society_id,society_key=society_key))
		else :
			user_output = UserModel().get_user_by_email(email,society_id,profile_member_type)
			if user_output : 
				user_id          = user_output["user_id"]
				encrypt_user_id  = Cryptography.encrypt(int(user_id))
				MailOtp(user_id,society_id,societyData,user_output)
				return redirect(url_for('user.getOtp',user_id=encrypt_user_id,society_id=society_id,society_key=society_key))
			else:
				flash("Sorry. We cannot find a membership record matching the given email id. For assistance, please write to  "+str(societyData['e_support_mail'])+". ","errorMsg")
				return redirect (url_for('user.getLogin',society_id=society_id,society_key=society_key)) 

@app.route('/<society_id>/<society_key>/at/<uuid>/<timestamp>', methods = ["GET", "POST"])
@RouteGroup.society_required
@RouteGroup.app_lock_requried
def autoLoginFromDashboard(society_id,society_key,uuid,timestamp):
	societyData    = Auth.getSociety(society_id)
	timestamp      = round(int(timestamp[:-3]))
	timestamp      = datetime.fromtimestamp(int(timestamp))
	if timestamp and (datetime.now() < timestamp):
		user_data  = UserModel().get_user_data_by_uuid(uuid)
		if user_data:
			Auth.login(user_data,society_id)
			# Auth.setSociety(societyData,society_id)
			return redirect(url_for('user.getProfile',society_id=society_id,society_key=society_key))
		else:
			flash("Sorry. We cannot find a membership record matching the given email id. For assistance, please write to  "+str(societyData['e_support_mail'])+". ","errorMsg")
			return redirect(url_for('user.getLogin',society_id=society_id,society_key=society_key))
	else:
		return redirect(url_for('user.getLogin',society_id=society_id,society_key=society_key))



@app.route('/<society_id>/<society_key>/at/app/', methods = ["GET", "POST"])
@app.route('/<society_id>/<society_key>/at/app/<uuid>', methods = ["GET", "POST"])
@RouteGroup.society_required
def autoLoginFromApp(society_id,society_key,uuid=None):
	r_url = request.values.get('r_url')
	societyData = Auth.getSociety(society_id)
	user_data   = UserModel().get_user_data_by_uuid(uuid)
	if user_data:
		Auth.login(user_data,society_id)
		# Auth.setSociety(societyData,society_id)
		return redirect(url_for('user.getProfile',society_id=society_id,society_key=society_key,r_url=r_url))
	else:
		return redirect(url_for('user.getLogin',society_id=society_id,society_key=society_key))
	
@app.route('<society_id>/<society_key>/logout', methods = ["GET", "POST"])
@RouteGroup.login_required
def Logout(society_id,society_key):
	Auth.logout(society_id)
	session.pop('_flashes', None)
	r_url = request.values.get('r_url')
	if r_url:
		r_url = r_url[1:-1]+"/logout"
		return redirect(r_url, code=302)
	else:	
		return redirect(url_for('user.getLogin',society_id=society_id,society_key=society_key))

""" END Login and Logout functions """
""" OTP LOGIN START """

def MailOtp(user_id,society_id,societyData,user=None):
	random.seed(time.process_time())
	now            = datetime.now()
	if user is None:
		user       = UserModel().get_otp_random(user_id)

	if user and user['otp']: 
		otp_random = user['otp']
	else :  
		otp_random  = Helper.random4digt()
	
	otp_sent_at     = now
	otp_expire_on   = now + timedelta(minutes=30)
	data            = {'otp_expire_on' : otp_expire_on,'otp_sent_at':otp_sent_at,'otp' : otp_random}
	output          = UserModel().update_member(user_id,data)
	user            = UserModel().get_member(user_id)

	if user :
		date            = otp_expire_on.strftime("%d/%B/%Y %I:%M %p")
		template_name   = "profile_otp_mail"
		mailData        = UserModel().get_mail_template(society_id,template_name)
		subject_data    = {'random_otp':str(otp_random),'society_name': societyData['society_name'],'date':date}
		subject         = Helper.merge_tags(mailData.subject,subject_data)
		users           = {'full_name':user['full_name'],'prefix':user['prefix'],'otp':otp_random,'date':date,'society_name': societyData['society_name'],'support_mail':societyData['e_support_mail']}
		mail_data       = {
							'html_content' : mailData.mail_content,
							'users'        : users
							}
		mail_content    = Helper.convert_html_from_string(mail_data)
		to          = user["email"]
		html        = render_template('users/email/base_email.html',mail_content=mail_content,societyData=societyData)
		# return(html)
		EMAIL.sendMail(subject,html,to,mailData,societyData)
	return "success"



@app.route('/<int:society_id>/<society_key>/gen_otp/<uuid>',methods = ["GET","POST"])
@RouteGroup.app_lock_requried
def gen_otp(society_id,society_key,uuid):
	societyData = UserModel().get_society_data(society_id)
	user_output = UserModel().get_user_data_by_uuid(uuid)
	if user_output : 
		return render_template('users/gen_otp.html',society_id=society_id,society_key=society_key,Data=societyData,user=user_output,uuid=uuid)
	else:
		flash("Sorry. We cannot find a membership record matching the given email id. For assistance, please write to  "+str(societyData['e_support_mail'])+". ","errorMsg")
		return redirect (url_for('user.getLogin',society_id=society_id,society_key=society_key)) 



@app.route('/<int:society_id>/<society_key>/otp/<uuid>',methods = ["GET","POST"])
@RouteGroup.app_lock_requried
def otp(society_id,society_key,uuid):
	societyData = UserModel().get_society_data(society_id)
	user_output = UserModel().get_user_data_by_uuid(uuid)
	if user_output : 
		user_id          = user_output["user_id"]
		encrypt_user_id  = Cryptography.encrypt(int(user_id))
		MailOtp(user_id,society_id,societyData,user_output)
		return redirect(url_for('user.getOtp',user_id=encrypt_user_id,society_id=society_id,society_key=society_key))
	else:
		flash("Sorry. We cannot find a membership record matching the given email id. For assistance, please write to  "+str(societyData['e_support_mail'])+". ","errorMsg")
		return redirect (url_for('user.getLogin',society_id=society_id,society_key=society_key)) 



@app.route('/<society_id>/<society_key>/otp_page/<user_id>', methods = ["GET","POST"])
@RouteGroup.society_required
def getOtp(user_id,society_id,society_key):
	user_id         = unquote(user_id)
	try:
		user_id         = int(Cryptography.decrypt(user_id))
		encrypt_user_id = Cryptography.encrypt(int(user_id))
		user_data       = UserModel().get_member(user_id)
		email           = user_data['email']
		societyData     = Auth.getSociety(society_id)
		flash("An OTP has been sent to your registered email id "+ Helper.email_mask(email),"successMsg")
		return render_template('users/otp_new.html',encrypt_user_id=encrypt_user_id,data=user_data,user_id=user_id,societyData=societyData,society_id=society_id,society_key=society_key)
	except Exception as e:
		logging.error(e)
		flash("Sorry, something went wrong. Please Login again","errormsg")
		return redirect(url_for('user.getLogin',society_id=society_id,society_key=society_key))


@app.route('/<society_id>/<society_key>/verify_otp/<user_id>', methods = ["GET","POST"])
@RouteGroup.society_required
def postOtp(user_id,society_id,society_key):
	try:
		user_id          = unquote(user_id)
		user_id          = int(Cryptography.decrypt(user_id))
		now              = datetime.now()
		otp              = request.values.get('input1',0) or 0
		userData         = UserModel().get_member(user_id)
		# otp  = userData['otp']
		encrypt_user_id  = Cryptography.encrypt(int(user_id))
		if userData["email"] == Helper.testmail and int(otp) == 9999 :
			Auth.login(userData,society_id)
			return redirect(url_for('user.getProfile',society_id=society_id,society_key=society_key))
		elif(int(otp) == userData['otp']) :
			data            = {'otp' : None, 'otp_verified_at':now}
			output          = UserModel().update_member(user_id,data)
			otp_expire_on   = datetime.strptime(str(userData["otp_expire_on"]),'%Y-%m-%d %H:%M:%S')
			if str(now)>str(otp_expire_on):
				flash("Expired!OTP valid for 30 mintue","errorMsg")
				return render_template('users/otp_new.html',encrypt_user_id=encrypt_user_id,user_id=user_id,society_id=society_id,society_key=society_key)
			else:
				Auth.login(userData,society_id)
				return redirect(url_for('user.getProfile',society_id=society_id,society_key=society_key))
		else:
			flash("Incorrect OTP.  Please provide the correct OTP","errorMsg")
			return render_template('users/otp_new.html',encrypt_user_id=encrypt_user_id,user_id=user_id,society_id=society_id,society_key=society_key)
	except Exception as e:
		logging.error(e)
		flash("Sorry, something went wrong. Please Login again","errorMsg")
		return redirect(url_for('user.getLogin',society_id=society_id,society_key=society_key))      


""" OTP LOGIN END """

@app.route('/<society_id>/<society_key>/resend_otp/<user_id>', methods = ["GET","POST"])
@RouteGroup.society_required
def resendOTP(user_id,society_id,society_key):
	try:
		societyData        = Auth.getSociety(society_id)
		user_id              = unquote(user_id)
		user_id              = int(Cryptography.decrypt(user_id))
		userData             = UserModel().get_member(user_id)
		if MailOtp(user_id,society_id,societyData,userData):
			encrypt_user_id  = Cryptography.encrypt(int(user_id))
			return redirect (url_for('user.getOtp',user_id=encrypt_user_id,society_id=society_id,society_key=society_key))
		else:
			flash("Something Wrong, Please Try Again","errorMsg")
			return redirect(url_for('user.getLogin',society_id=society_id,society_key=society_key))
	except Exception as e:
		logging.error(e)
		flash("Sorry, something went wrong. Please Login again","errorMsg")
		return redirect(url_for('user.getLogin',society_id=society_id,society_key=society_key))      

@app.route('/<society_id>/<society_key>/profile',methods = ["GET"])
@RouteGroup.society_required
@RouteGroup.login_required
def getProfile(society_id,society_key,user_data=None):
#   user_id     = unquote(user_id)
#   user_id     = int(Cryptography.decrypt(user_id))
	r_url = request.values.get('r_url')
	user_id     = Auth.user(society_id).get('user_id')
	member_type_id = Auth.user(society_id).get('member_type_id')
	attach_path = Auth.user(society_id).get('attach_path')
	societyData = Auth.getSociety(society_id)
	# societyData = UserModel().get_society(society_id,society_key)
	# attch_type_id = Helper.attch_type_id[society_key]
	attch_type_id  = societyData['attach_type_id']
	state       = UserModel().get_state()
	district    = UserModel().get_districts()
	country     = UserModel().get_country()
	is_confirm = 0
	# img = UserModel().get_fellowship_data(user_id)
	if (int(society_id))==10:
		specialities      = UserModel().get_specialities(user_id)
	elif (int(society_id)==2):
		specialities      = UserModel().get_longterm_specialities()
	else:
		specialities      = None
	if attch_type_id:
		if user_data is None:
			user_data   = UserModel().get_member_attachment(user_id,attch_type_id)
			if int(society_id) in (1,18):
				if int(society_id) == 1:
					attach_type_id = 11
				if int(society_id) == 18:
					attach_type_id = '50,68'
				upload_details = UserModel().get_upload_details(society_id,attach_type_id)
				img = UserModel().get_fellowship_data(user_id,attach_type_id)
				fellowship_data = render_template('users/fellowship_certificate.html',upload_details=upload_details,img=img,societyData=societyData)
				return render_template('users/profile_update.html',data=user_data,img=img,state=state,district=district,country = country,Data=societyData,society_id=society_id,society_key=society_key,fellowship_data=fellowship_data,is_confirm=is_confirm,member_type_id=member_type_id,specialities=specialities,r_url=r_url,datetime=datetime)
			elif int(society_id)==10:
				attach_type_id = 67
				upload_details = UserModel().get_upload_details(society_id,attach_type_id)
				img = UserModel().get_fellowship_data(user_id,attach_type_id)
				govt_proof = render_template('users/govt_proof.html',upload_details=upload_details,img=img,societyData=societyData)
				return render_template('users/profile_update.html',data=user_data,img=img,state=state,district=district,country = country,Data=societyData,society_id=society_id,society_key=society_key,govt_proof=govt_proof,is_confirm=is_confirm,member_type_id=member_type_id,specialities=specialities,r_url=r_url,datetime=datetime)
			elif int(society_id) == 2:
				if user_data['longterm_specialities']:
					longterm_specialities = user_data['longterm_specialities'].split(",")
				else:
					longterm_specialities = None
				if user_data['shortterm_specialities']:
					shortterm_specialities = user_data['shortterm_specialities'].split(",")
				else:
					shortterm_specialities=None
				return render_template('users/mos_profile_update.html',data=user_data,state=state,district=district,country = country,Data=societyData,society_id=society_id,society_key=society_key,is_confirm=is_confirm,member_type_id=member_type_id,specialities=specialities,r_url=r_url,datetime=datetime,longterm_specialities=longterm_specialities,shortterm_specialities=shortterm_specialities)
			elif int(society_id) == 36:
				attach_type_id = '75,76,79'
				upload_details = UserModel().get_upload_details(society_id,attach_type_id)
				img = UserModel().get_fellowship_data(user_id,attach_type_id)
				fellowship_data = render_template('users/yosi_certificates.html',upload_details=upload_details,img=img,societyData=societyData,society_id=society_id)
				return render_template('users/yosi_profile_update.html',data=user_data,img=img,fellowship_data=fellowship_data,state=state,district=district,country = country,Data=societyData,society_id=society_id,society_key=society_key,is_confirm=is_confirm,member_type_id=member_type_id,r_url=r_url,datetime=datetime)
			else:
				return render_template('users/profile_update.html',data=user_data,state=state,district=district,country = country,Data=societyData,society_id=society_id,society_key=society_key,is_confirm=is_confirm,member_type_id=member_type_id,specialities=specialities,r_url=r_url,datetime=datetime)
		else:
			user_data=user_data
			if int(society_id) in (1,18):
				if int(society_id) == 1:
					attach_type_id = 11
				if int(society_id) == 18:
					attach_type_id = 49
				upload_details = UserModel().get_upload_details(society_id,attach_type_id)
				img = UserModel().get_fellowship_data(user_id,attach_type_id)
				fellowship_data = render_template('users/fellowship_certificate.html',upload_details=upload_details,img=img,societyData=societyData)
				return render_template('users/profile_update.html',data=user_data,state=state,district=district,country = country,Data=societyData,society_id=society_id,society_key=society_key,fellowship_data=fellowship_data,is_confirm=is_confirm,img=img,member_type_id=member_type_id,specialities=specialities,r_url=r_url,datetime=datetime)
			elif int(society_id)==10:
				attach_type_id = 67
				upload_details = UserModel().get_upload_details(society_id,attach_type_id)
				img = UserModel().get_fellowship_data(user_id,attach_type_id)
				govt_proof = render_template('users/govt_proof.html',upload_details=upload_details,img=img,societyData=societyData)
				return render_template('users/profile_update.html',data=user_data,img=img,state=state,district=district,country = country,Data=societyData,society_id=society_id,society_key=society_key,govt_proof=govt_proof,is_confirm=is_confirm,member_type_id=member_type_id,specialities=specialities,r_url=r_url,datetime=datetime)
			elif int(society_id == 2):
				return render_template('users/mos_profile_update.html',data=user_data,state=state,district=district,country = country,Data=societyData,society_id=society_id,society_key=society_key,is_confirm=is_confirm,member_type_id=member_type_id,specialities=specialities,r_url=r_url,datetime=datetime)
			elif int(society_id) == 36:
				attach_type_id = '75,76,79'
				upload_details = UserModel().get_upload_details(society_id,attach_type_id)
				img = UserModel().get_fellowship_data(user_id,attach_type_id)
				fellowship_data = render_template('users/yosi_certificates.html',upload_details=upload_details,img=img,societyData=societyData,society_id=society_id)
				return render_template('users/yosi_profile_update.html',data=user_data,img=img,fellowship_data=fellowship_data,state=state,district=district,country = country,Data=societyData,society_id=society_id,society_key=society_key,is_confirm=is_confirm,member_type_id=member_type_id,r_url=r_url,datetime=datetime)
			else:
				return render_template('users/profile_update.html',data=user_data,state=state,country = country,district=district,Data=societyData,society_id=society_id,society_key=society_key,is_confirm=is_confirm,member_type_id=member_type_id,specialities=specialities,r_url=r_url,datetime=datetime)
	else:
		return 'No Attach Type Id'
	if not user_data:
		return "<html><h1>PAGE NOT FOUND</h1><h4>Sorry, the page you were looking for could not found. Please check the URL and try your luck again.</h4></html>"
	state       = UserModel().get_state()
	country     = UserModel().get_country()
	return render_template('users/profile_update.html',data=user_data,state=state,country = country,Data=societyData,society_id=society_id,society_key=society_key,r_url=r_url,datetime=datetime)



@app.route('/<society_id>/<society_key>/profile',methods=["POST"])
@RouteGroup.society_required
@RouteGroup.login_required
def postProfile(society_id,society_key):
	if request.method == "POST":
		r_url = request.values.get('r_url')
		user         = Auth.user(society_id)
		user_id      = request.values.get("user_id",None)
		participation_id      = request.values.get("participation_id",0)
		district     = request.values.get("district",None)
		db_full_name = request.values.get("db_full_name", None)
		longterm_specialities = request.values.getlist('longterm_specialities',None)
		shortterm_specialities = request.values.getlist('shortterm_specialities',None)
		if longterm_specialities:
			longterm_specialities = (",".join(longterm_specialities))
		else:
			longterm_specialities = None
		if shortterm_specialities:
			shortterm_specialities = (",".join(shortterm_specialities))
		else:
			shortterm_specialities = None
		if int(society_id)== 10:
			mc_number    = request.form.get("mc_no",None)
		else:
			mc_number    = request.form.get("mc_number",None)
		# is_confirm   = request.values.get("is_confirm") or 0
		data={
			'prefix'      : request.form.get("prefix",None),
#        'full_name'   : request.form.get("name",None),
#        'full_name'   : user.get("full_name"),
			'mobile'      : request.form.get("mobile",None),
			'email'       : user.get("email"),
			'gender'      : request.form.get("gender",None),
			'dob'         : (request.form.get("year",None)+"-"+request.form.get("month",None)+"-"+request.form.get("date",None)),
			'aios_membership_number'   : request.form.get("aios_membership_number",None),
			'mc_number'   : mc_number,
			'mc_state_id' : request.values.get("mc_state",None),
			'phone_home'  : request.form.get("phone_home",None),
			'phone_office': request.form.get("phone_office",None),
			'address1'    : request.form.get("Address1",None),
			'address2'    : request.form.get("Address2",None),
			'address3'    : request.form.get("Address3",None),
			'state_id'    : request.values.get("state",None),
			'country_id'  : request.values.get("country",None),
			'city'        : request.values.get("city",None),
			'district'    : district,
			'other_districts'  : (request.values.get("other_district") or None) if district == "Others" else None ,
			'pincode'     : request.form.get("pincode",None),
			'profile_updated_at'  : datetime.now(),
			'whatsapp_number': request.form.get("whatsapp_number",None),
			'com_address1'    : request.form.get("c_address1",None),
			'com_address2'    : request.form.get("c_address2",None),
			'com_address3'    : request.form.get("c_address3",None),
			'com_state_id'    : request.values.get("c_state",None),
			'com_country_id'  : request.values.get("c_country",None),
			'com_city'        : request.values.get("c_city",None),
			'com_district'    : request.values.get("c_district",None),
			'com_pincode'     : request.form.get("c_pincode",None),
			'rejected_for_life_membership': None,
			'gov_proof_type' : request.values.get("gov_proof_type",None),
			'gov_proof_id' : request.values.get("gov_proof_id",None), 
			'age' : request.values.get("age",None), 
			'hos_name'     : request.form.get("hos_name",None),
			
		}
		data1={
			'prefix'      : request.form.get("prefix",None),
			'full_name'   : request.form.get("name",None),
			'mobile'      : request.form.get("mobile",None),
			'email'       : user.get('email'),
			'gender'      : request.form.get("gender",None),
			'dob'         : (request.form.get("year",None)+"-"+request.form.get("month",None)+"-"+request.form.get("date",None)),
			'mc_number'   : request.form.get("mc_number",None),
			'mc_state_id' : request.values.get("mc_state",None),
			'address1'    : request.form.get("Address1",None),
			'address2'    : request.form.get("Address2",None),
			'address3'    : request.form.get("Address3",None),
			'state_id'    : request.values.get("state",None),
			'country_id'  : request.values.get("country",None),
			'city'        : request.values.get("city",None),
			'district'    : request.values.get("district",None),
			'pincode'     : request.form.get("pincode",None),
		}

		data2 = {
			'user_id'       : request.values.get("user_id",None),
			'is_speaker'    : request.values.get("is_speaker",None),
			'is_moderator'  : request.form.get("is_moderator",None),
			'society_id'    : request.form.get("society_id",None),
		}

		data3 = {
			'user_id'       : request.values.get("user_id",None),
			'place_of_practice'     : request.form.get("place_of_practice",None),
			'district_of_practice'     : request.form.get("district_of_practice",None),
			'is_teaching_institute'     : request.form.get("is_teaching_institute",None),
			'is_government_institute'     : request.form.get("is_government_institute",None),
			'longterm_specialities'     : longterm_specialities,
			'year_of_longterm'     : request.form.get("year_of_longterm",None),
			'institute_of_longterm'     : request.form.get("institute_of_longterm",None),
			'place_of_longterm'     : request.form.get("place_of_longterm",None),
			'year_of_experience'     : request.form.get("year_of_experience",None),
			'year_of_experience_post_fellowship'     : request.form.get("year_of_experience_post_fellowship",None),
			'shortterm_specialities'     : shortterm_specialities,
			'year_of_shortterm'     : request.form.getlist("year_of_shortterm",None),
			'institute_of_shortterm'     : request.form.get("institute_of_shortterm",None),
			'place_of_shortterm'     : request.form.get("place_of_shortterm",None),
			'user_topics'     : request.form.get("user_topics",None),
			'additional_info'     : request.form.get("additional_info",None),
			'year_of_passing'     : request.form.get("year_of_passing",None),
			'institution'     : request.form.get("institution",None),
		}

		data            = additional_data(society_id,data)
		state       = UserModel().get_state()
		country     = UserModel().get_country()
		societyData = Auth.getSociety(society_id)
		# societyData = UserModel().get_society(society_id,society_key)
		
		# attch_type_id = Helper.attch_type_id[society_key]
		attch_type_id   = societyData['attach_type_id']
		user_data       = UserModel().get_member_attachment(user_id,attch_type_id)

		if (int(society_id))== 10:
			specialities_list    = request.form.getlist('chk_specialities')
			specialities_insert_delete(specialities_list,user_id)
		
		file            = request.files['files']
		data['dob'] = datetime.strptime(data['dob'], '%Y-%m-%d')
		if db_full_name=='None':
			data["full_name"] = request.form.get("name",None)
		whats_app_dupli = whats_app_duplicate(data["whatsapp_number"],user_id,society_id)
		if whats_app_dupli == 1:
			flash("Whatsapp number "+ data["whatsapp_number"] +" already exists with us.We cannot use same number","errorMsg")
			user_id     = Cryptography.encrypt(int(user_id))
			data        = Helper.join_dicts(user_data, data)    
			# dob = datetime(int(request.form.get("year",0)),int(request.form.get("month",0)),int(request.form.get("date",0)))
			
			# return render_template('users/profile_update.html',data=data,state=state,country = country,Data=societyData,society_id=society_id,society_key=society_key)
			# return redirect(url_for('user.getProfile',society_id=society_id,society_key=society_key,user_data=data))
			return getProfile(society_id=society_id,society_key=society_key,user_data=data)
		else:
#           Log().insert({'level':'INFO','message':str(user_id)})
#           Log().insert({'level':'INFO','message':str(society_id)})
			mobile_count = UserModel().get_mobile_count(data["mobile"],user_id,society_id)
			if mobile_count > 0 :
				flash("Mobile number "+ data["mobile"] +" already exists with us.We cannot use same number","errorMsg")
				data        = Helper.join_dicts(user_data, data)    
				# data['dob'] = datetime(int(request.form.get("year",0)),int(request.form.get("month",0)),int(request.form.get("date",0)))
				return getProfile(society_id=society_id,society_key=society_key,user_data=data)
				# return redirect(url_for('user.getProfile',society_id=society_id,society_key=society_key,data=data))
				# return render_template('users/profile_update.html',data=data,state=state,country = country,Data=societyData,society_id=society_id,society_key=society_key)
			else :
				UserModel().update_member(user_id,data)
				if int(society_id)==2:
					user_additional_data = UserModel().get_user_additional(user_id)
					if user_additional_data:
						UserModel().update_user_additional(user_id,data3)
					else:
						UserModel().insert_user_additional(data3)
				if int(society_id) == 17:
					output          = UserModel().update_delegate_data(user_id,data1)
				elif int(society_id) == 10:
					output = UserModel().update_or_insert_participation(user_id,participation_id,data2)
					# output          = UserModel().insert_user_participation(data2)
				else :
					pass
			if r_url:
				return redirect(r_url[1:-1], code=302)
			# flash('Profile has successfully updated')
			return redirect(url_for('user.mailConfirmation',society_id=society_id,society_key=society_key,user_id = user_id))
	else:
		flash('Session Expired')
		return redirect(url_for('user.getLogin',society_id=society_id,society_key=society_key))
	
def specialities_insert_delete(specialities_list,user_id):
	old_specility_data   = UserModel().get_old_specialities(user_id)
	new_specialities  = specialities_list
	insert_specialities = new_specialities
	delete_specialities = []
	if old_specility_data:
		for j in old_specility_data:  
			delete_specialities.append(str(j["category_id"]))
			if str(j["category_id"]) in new_specialities:
				delete_specialities.remove(str(j["category_id"]))
				insert_specialities.remove(str(j["category_id"]))

	if new_specialities:
		insert_specialities = new_specialities
		created_at = datetime.now()
		UserModel().insert_specialities_db(insert_specialities,user_id,created_at)
	if delete_specialities:
		delete_specialities = ",".join(delete_specialities)
		UserModel().delete_specialities_db(delete_specialities,user_id)
	return "success"

@app.route('/profile_pic_upload', methods = ["GET","POST"])                                     
def ProfilePicUpload():
	if request.method == 'POST':
		user_id         = request.values.get('user_id')
		image_data = request.form['image_data']
		file_name       = request.form['file_name']
		society_id      = request.form['society_id']
		society_key     = request.form['society_key'] 
		user            = Auth.user(society_id)
		societyData     = Auth.getSociety(society_id)
		# societyData = UserModel().get_society(society_id,society_key)
		if image_data :
			acl             ="public-read"
			s3              = boto3.client("s3",aws_access_key_id='AKIAILLDAQ7NFUMVOYLQ',aws_secret_access_key='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')
			split_tup       = os.path.splitext(file_name)
			file_name       = split_tup[0]
			file_extension  = split_tup[1]
			file_str        = "Photograph"+file_extension
			if " " in file_str :
				file_str   = file_str.replace(" ","")
			else :
				pass

			try:
				bucket     = Auth.getSociety(society_id).get("aws_bucket")
				if bucket == 'common-application':
					foldername = str(society_key)+"/user_attachment"
				else:
					foldername = "user_attachment" 

				if image_data: 
					image_data = base64.b64decode(image_data.split(',')[1])
					image      = Image.open(io.BytesIO(image_data))
					if image.mode == 'RGBA':
						image = image.convert('RGB')
					image.save(os.path.join('core/static/temp_images', file_str))
					img_path = os.path.join('core/static/temp_images/'+ file_str)
					with open(img_path, 'rb') as f:
						response = s3.upload_fileobj(f,bucket,foldername+'/'+user_id+'/'+ file_str, ExtraArgs={"ACL": acl, "ContentType": "image/jpeg"})        
					os.remove('core/static/temp_images/'+ file_str) 
					file_path  = "https://"+bucket+".s3-ap-southeast-1.amazonaws.com/"+foldername+"/"+str(user_id)
					data_1     = {
								'attach_path'      : file_path,
								'attach_file_name' : file_str,
								'attach_type_id'   : societyData['attach_type_id'],
								 }
					update_data = UserModel().update_attachemant(user_id,data_1,str(datetime.now()))
					CallDelegateUpdateAPI(society_id,user_id,file_path,file_str)

				return json.dumps(data_1)
					# response   = s3.upload_fileobj(file, bucket,foldername+'/'+str(user_id)+'/'+ file_str, ExtraArgs={"ACL": acl,"ContentType": file.content_type})
			except ClientError as e:
				logging.error(e)
				flash("Image upload failed.","errorMsg")
				user_id     = Cryptography.encrypt(int(user_id))
				return redirect(url_for('user.getProfile',society_id=society_id,society_key=society_key))
		else :
			pass

	return"success" 


def whats_app_duplicate(whatsapp_no,user_id,society_id):
	if whatsapp_no:
		count_whatsapp_num = UserModel().count_whatsapp_num(whatsapp_no,user_id,society_id)
		if count_whatsapp_num > 0:
			return 1
		else:
			return "success"
	else:
		return "success"


def additional_data(society_id, data):
	data1 = {}
	if int(society_id) in [1,18]: #VRSI and KSOS
		data1 = {
			'hos_name'          :request.values.get("hos_name",None),
			'hos_address1'      :request.values.get("hos_address1",None),
			'hos_address2'      :request.values.get("hos_address2",None),
			'hos_address3'      :request.values.get("hos_address3",None),
			'hos_city'          :request.values.get("hos_city",None),
			'hos_state'         :request.values.get("hos_state",None),
			'hos_email'         :request.values.get("hos_email",None),
			'hos_contact'       :request.values.get("hos_contact",None),
			'hos_whatsappnumber':request.values.get("hos_whatsappnumber",None),
			'institution'       : request.values.get("institution",None),
			'hos_profile_pic_visible': request.values.get("hos_profile_pic_visible",None)
		}
		
	if int(society_id) in [18]: #KSOS
		user_id    = request.values.get("user_id",None)
		is_public = request.values.get("is_public",None)
		user_data  = UserModel().get_member(user_id)
		prev_opt   =user_data["e_journal_opt_in"] 
		e_journal_opt_in = request.values.get("e_journal_opt_in")
		data1['is_public']= is_public
		if int(prev_opt) == int(e_journal_opt_in):
			e_journal_opt_in=prev_opt
		else:
			e_journal_opt_in=e_journal_opt_in

		data1['e_journal_opt_in']= e_journal_opt_in
		
		
	if int(society_id) == 10: #KOS
		role_of_faculty = request.form.getlist('role_of_faculty_you_are_interested')
		role_of_faculty_you_are_interested=','.join(role_of_faculty)
		data1 = {
			# 'gov_proof_type' : request.values.get("gov_proof_type",None),
			# 'gov_proof_id' : request.values.get("gov_proof_id",None),
			'participate_in_next_con' : request.values.get("participate_in_next_con",None),
			'participate_as_faculty_in_next_con' : request.values.get("participate_as_faculty_in_next_con",None),
			'role_of_faculty_you_are_interested' : role_of_faculty_you_are_interested,
			'topics_you_can_deliver' : request.values.get("topics_you_can_deliver",None),
			'Interested_to_evaluate' : request.values.get("Interested_to_evaluate",None),
			'your_core_specialities' : request.values.get("your_core_specialities",None),
			'experience'  : request.values.get("experience",None),
			'type_of_practice'  : request.values.get("type_of_practice",None), 
		}
		
	if int(society_id) == 3:
		data1 = {
			'gov_proof_type' : request.values.get("gov_proof_type",None),
			'gov_proof_id' : request.values.get("gov_proof_id",None),
		}
		
	
	return Helper.join_dicts(data, data1)
		

@app.route('/<society_id>/<society_key>/mail_confirmation/<user_id>',methods=["GET","POST"])
@RouteGroup.society_required
def mailConfirmation(user_id,society_id,society_key):
	societyData     = Auth.getSociety(society_id)
	now             = datetime.now()
	current_dt      = now
	date            = current_dt.strftime("%d/%B/%Y %I:%M %p")
	# attch_type_id   = Helper.attch_type_id[society_key]
	attch_type_id   = societyData['attach_type_id']
	user            = UserModel().get_member_attachment(user_id,attch_type_id)
	template_name   ="profile_confirmation_mail"
	mailData        = UserModel().get_mail_template(society_id,template_name)
	subject_data    ={'prefix': user['prefix'],'full_name': user['full_name'],'date':date}
	subject         = Helper.merge_tags(mailData.subject,subject_data)
	users           = {'full_name':user['full_name'],'prefix':user['prefix'],'user_uuid':user['user_uuid'],'member_type_id':user['member_type_id']}
	if (int(society_id) in [1,18]):
		if int(society_id) == 1:
			attach_type_id = 11
		if int(society_id) == 18:
			attach_type_id = '50,68'
		fellowship = UserModel().get_fellowship_data(user_id,attach_type_id)
	if (int(society_id) in [10]):
		attach_type_id = 67
		specialites = UserModel().get_specialites_data(user_id)
		fellowship = UserModel().get_fellowship_data(user_id,attach_type_id)
		user_participation = UserModel().get_user_participation_data(user_id,society_id)
	
	
	
	address_list = []
	if user['address1'] or None:
		address_list.append(user['address1']+",")
	if user['address2'] or None:
		address_list.append(user['address2']+",") 
	if user['address3'] or None:
		address_list.append(user['address3']+",") 
	if user['city'] or None:
		address_list.append(user['city']+",") 
	if user['district']=="Others":
		address_list.append(user['other_districts']+",")
	else:
		address_list.append(user['district']+",")
	# if user['district'] or None:
	#     address_list.append(user['district']+",") 
	if user['state_name'] or None:
		address_list.append(user['state_name']) 
	if user['country_name'] or None:
		address_list.append(user['country_name']) 
	if user['pincode'] or None:
		address_list.append(user['pincode']) 
	
				  
				  
	mail_data       ={
			'html_content':mailData.mail_content,
			'user'        :users,
			'societyData' :societyData,
			'data'        :user,
			'address_list':address_list,
			'fellowship'  : fellowship if int(society_id) in (1,18,10) else "" ,
			'specialites' : specialites if int(society_id)==10 else "",
			'user_participation' : user_participation if int(society_id)==10 else ""
	}
	mail_content     = Helper.convert_html_from_string(mail_data)
	to               = user["email"]
	# subject = user["prefix"]+user["full_name"]+", Your profile has been updated on "+datetime.now().strftime("%d-%m-%Y %H:%M:%S")+"."
	html             = render_template('users/email/confirmation_mail.html',mail_content=mail_content,societyData=societyData,data=user)
	# return html
	# EMAIL.sendMail(subject,html,email)
	# to="mukeshkumar@numerotec.com"
	cc               = []
	bcc              = []
	EMAIL.sendMail(subject,html,to,mailData,societyData)
	return redirect(url_for("user.getSuccess",society_id=society_id,society_key=society_key))


@app.route('/get_state',methods=["GET","POST"])
def getState():
	country_id = request.values.get("country_id") or None
	state_data = None
	if country_id:
		state_data = UserModel().get_country_data(country_id)
	return jsonify(state_data if state_data else '')

@app.route('/<society_id>/<society_key>/updatesuccess',methods=["GET","POST"])
@RouteGroup.society_required
# @RouteGroup.login_required
def getSuccess(society_id,society_key):
#   Auth.logout(society_id)
	societyData = Auth.getSociety(society_id)
	user_data = Auth.user(society_id)
	apps = UserModel().get_abs_and_del_data(society_id)
	return render_template('users/success.html',societyData=societyData,user_data=user_data,society_id=society_id,society_key=society_key,apps=apps)
	
@app.route('/<society_id>/<society_key>/verify_profile/<uuid>',methods=["GET","POST"])
def postVerifyProfile(society_id,society_key,uuid):
	data       = UserModel().get_user_data_by_uuid(uuid)
	user_id=data['user_id']
	if user_id:
		data={
			'profile_updated_at'  : datetime.now(),
		}
		update_data  = UserModel().update_member(user_id,data)
		return redirect(url_for('user.getSuccess',society_id=society_id,society_key=society_key))
	else:
		flash('Session Expired')
		return redirect(url_for('user.getLogin',society_id=society_id,society_key=society_key))
	
@app.route('/post_amzon_upload/<society_id>/<society_key>', methods = ["GET","POST"])
def PostAmazonUpload(society_id,society_key,acl="public-read"):
	if request.method == 'POST':
		user_id         = request.values.get('user_id')
		attach_id       = request.values['attach_id'] 
		attach_type_id  = request.values['attach_type_id']
		# attach_id       = request.values.get('attach_id')
		# attach_type_id  = request.values.get('attach_type_id')
		societyData    = Auth.getSociety(society_id)
		s3             = boto3.client("s3",aws_access_key_id='AKIAILLDAQ7NFUMVOYLQ',aws_secret_access_key='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')
		files          = request.files.getlist("file")
		for file in files:
			if file.filename:
				filename       = secure_filename(file.filename)
				split_tup       = os.path.splitext(filename)
				file_name       = split_tup[0]
				file_extension  = split_tup[1]
				datas          = UserModel().get_upload_details(society_id,attach_type_id)
				attach_type    = datas[0]['attach_type']
				file_str        = attach_type+file_extension
				if " " in file_str :
					file_str = file_str.replace(" ","")
				try:
					bucket     = societyData['aws_bucket']
					if bucket == 'common-application':
						foldername = str(society_key)+"/user_attachment"
					else:
						foldername = "user_attachment"
					user_id    = str(user_id)
					
					response   = s3.upload_fileobj(file, bucket,foldername+'/'+user_id+'/'+ file_str, ExtraArgs={"ACL": acl,"ContentType": file.content_type})
					data_1     = {'bucketname' : societyData['aws_bucket'],'foldername':foldername,'file_name':file_str+"?"+str(Helper.getcurrenttimestamp()),'msg':'Image successfully uploded','file_extension':file_extension,'status':1} 
					file_path  = societyData['aws_url']+foldername+"/"+user_id
					data = {
						'user_id'           : user_id,
						'attach_type_id'    : attach_type_id,
						'attach_file_name'  : file_str,
						'attach_path'       : file_path,
						'created_at'        : datetime.now(),
						'updated_at'        : datetime.now()  
					}
					if (int(attach_id) > 0):
						ua  = UserModel().get_user_attach(attach_id,attach_type_id) #UserModel().get_fellowship_data(user_id)
						if ua:
							data.pop("created_at") 
							data        = UserModel().update_image(attach_id,data)
							d           = {}
							d['attach_id']  = attach_id
							data_1.update(d)
					else :
						attach_id_pk   = UserModel().insert_image(data)
						d              = {}
						d['attach_id'] = attach_id_pk
						data_1.update(d)
					return json.dumps(data_1)
				except ClientError as e:
					logging.error(e)
					print(logging.error(e))
					data_1    = {'msg':'Image upload failed .Please try again.','status':0}        
					json_data = json.dumps(data_1)
					return json_data
					# flash('Something Error .Please Try Again')
	return "success"

@app.route('/delete_amazon_image/<society_id>/<society_key>', methods = ["GET","POST"])
def DeleteAmazonImage(society_id,society_key):
	attach_id       = request.form['attach_id']
	attach_type_id  = request.form['attach_type_id']
	societyData    = Auth.getSociety(society_id)
	ua              = UserModel().get_user_attach(attach_id,attach_type_id)
	if ua is None:
		data = {'msg':'Deleted','status' : 1}
	else:
		filepath        = ua['attach_path']
		filename        = ua['attach_file_name']
		foldername      = filepath+ "/"+filename
		foldername      = foldername.replace(societyData['aws_url'], "")
		s3              = boto3.resource("s3",aws_access_key_id='AKIAILLDAQ7NFUMVOYLQ',aws_secret_access_key='EEudWCsWeCIt/J9/z5m5htmUdGFJNzWFPvNQBIp/')
		response        =  s3.Object(societyData['aws_bucket'], foldername).delete()
		UserModel().delete_image(attach_id,attach_type_id)
		data  = {'msg':'Deleted','status' : 1}
	json_data = json.dumps(data)
	return json_data

@app.route('/update_confirm_date/<society_id>/<society_key>', methods = ["GET","POST"])
def UpdateConfirmDate(society_id,society_key):
	user_id       = request.values.get('user_id')
	attach_id       = request.values.get('attach_id')
#  confirm_date   = datetime.now()
	confirm_date   = None
	UserModel().update_confirm_date(confirm_date,attach_id)
	data  = {'msg':'Updated','status' : 1}
	json_data = json.dumps(data)
	return json_data

# sqllite	
@app.route("/generate_session_data_by_sqlite", methods=["GET", "POST"])
def GenerateSqlite():
	UserModel().session_data_InsertToSqlite_1()
	return "success"
	
# for user attachment
@app.route("/api/profile_photo/<society_id>/<user_id>", methods=["POST"])
def UserAttachment(society_id, user_id):
	societyData 		= UserModel().get_society(society_id,None,None)
	attach_type_id      = UserModel().get_society_by_id(society_id)
	if societyData:
		attach_file_name 	= request.values.get('attach_file_name')
		attach_path 		= request.values.get('attach_path')
		url 				= str(attach_path)+"/"+str(attach_file_name)
		image_data 			= "," + Helper.ImageDecodeBase64(url)
		file_extension 		= Helper.GetExtensionFileUrl(attach_file_name)
		if image_data :
			acl             ="public-read"
			s3              = boto3.client("s3",aws_access_key_id='AKIAILLDAQ7NFUMVOYLQ',aws_secret_access_key='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')
			# split_tup       = os.path.splitext(file_name)
			# file_name       = split_tup[0]
			# file_extension  = split_tup[1]
			file_str        = "Photograph"+file_extension
			if " " in file_str :
				file_str   = file_str.replace(" ","")
			else :
				pass

			try:
				bucket     = societyData.get("aws_bucket")
				if bucket == 'common-application':
					foldername = str(societyData.get('society_key'))+"/user_attachment"
				else:
					foldername = "user_attachment" 

				if image_data:
					# print(">>>",image_data)
					image_data = base64.b64decode(image_data.split(',')[1])
					image      = Image.open(io.BytesIO(image_data))
					if image.mode == 'RGBA':
						image = image.convert('RGB')
					image.save(os.path.join('core/static/temp_images', file_str))
					img_path = os.path.join('core/static/temp_images/'+ file_str)
					with open(img_path, 'rb') as f:
						response = s3.upload_fileobj(f,bucket,foldername+'/'+user_id+'/'+ file_str, ExtraArgs={"ACL": acl, "ContentType": "image/jpeg"})        
					os.remove('core/static/temp_images/'+ file_str) 
					file_path  = "https://"+bucket+".s3-ap-southeast-1.amazonaws.com/"+foldername+"/"+str(user_id)
					data_1     = {
								'attach_path'      : file_path,
								'attach_file_name' : file_str,
								'attach_type_id'   : attach_type_id['photograph_attach_type_id'],
								 }
					update_data = UserModel().update_attachemant(user_id,data_1,str(datetime.now()))
					# CallDelegateUpdateAPI(society_id,user_id,file_path,file_str)
				return jsonify({'status':1,'msg':'success'})
				# response   = s3.upload_fileobj(file, bucket,foldername+'/'+str(user_id)+'/'+ file_str, ExtraArgs={"ACL": acl,"ContentType": file.content_type})
			except ClientError as e:
				logging.error(e)
				# flash("Image upload failed.","errorMsg")
				return jsonify({'status':0,'msg':'Image upload failed'})
		else:
			return jsonify({'status':0,'msg':'Photo url invalid'})	
	else:
		return jsonify({'status':0,'msg':'invalid soceity id'})
	

		
def CallDelegateUpdateAPI(society_id,user_id,file_path,file_str):
	url = f"https://delegate.numerotech.com/api/update_delegate_image"

	payload = {'del_img_filename': file_str,'del_img_path': file_path}
	files=[]
	headers = {}

	response = requests.request("POST", url, headers=headers, data=payload, files=files)

	print(response.text)
	return "success"

